from typing import List, Optional

from sziszapangma.core.transformer.embedding_transformer import \
    EmbeddingTransformer
from sziszapangma.core.wer.distance_matrix_calculator import \
    BinaryDistanceCalculator, DistanceCalculator, CosineDistanceCalculator
from sziszapangma.core.wer.step_words import StepWords
from sziszapangma.core.wer.wer_calculator import WerCalculator
from sziszapangma.core.wer.wer_processing_step import WerProcessingStep
from sziszapangma.core.wer.wer_step import WerStep


class WerEmbeddingCalculator(WerCalculator):
    _distance_calculator: DistanceCalculator

    def __init__(self, embedding_transformer: EmbeddingTransformer):
        super().__init__(BinaryDistanceCalculator())
        self._embedding_transformer = embedding_transformer
        self._distance_calculator = CosineDistanceCalculator(
            embedding_transformer)

    def _calculate_distance_for_word_step(
        self,
        step_words: StepWords
    ) -> float:
        return self._distance_calculator.calculate_distance_for_words(
            step_words.reference_word,
            step_words.hypothesis_word
        )

    def _calculate_result_cost_for_step(
        self,
        processing_step: WerProcessingStep
    ) -> float:
        step_words = processing_step.step_words
        return self._calculate_distance_for_word_step(step_words) \
            if processing_step.step_type.is_cross_step() \
            else processing_step.step_cost

    def convert_processing_steps_to_result(
        self,
        processing_steps: List[WerProcessingStep],
        reference_weights: Optional[List[float]] = None
    ) -> List[WerStep]:
        if reference_weights is None:
            return [
                WerStep(step.step_type, step.step_words,
                        self._calculate_result_cost_for_step(step))
                for step in processing_steps
            ]
        else:
            indexes_per_steps = self._get_reference_indexes_per_steps(
                processing_steps)
            return [
                WerStep(
                    processing_steps[step_index].step_type,
                    processing_steps[step_index].step_words,
                    reference_weights[indexes_per_steps[step_index]] *
                    self._calculate_result_cost_for_step(
                        processing_steps[step_index])
                )
                for step_index in range(len(processing_steps))
            ]
