from typing import List, Optional

import numpy as np
import pandas as pd

from sziszapangma.core.wer.wer_step import WerStep


class WerUtil:

    @staticmethod
    def _optional_str_to_str(value: Optional[str]) -> str:
        return value if value is not None else ''

    @staticmethod
    def _wer_step_to_pandas_row_lit(step: WerStep) -> List[any]:
        return [
            step.step_type.get_short_name(),
            WerUtil._optional_str_to_str(step.step_words.reference_word),
            WerUtil._optional_str_to_str(step.step_words.hypothesis_word),
            round(step.step_cost, 3)
        ]

    @staticmethod
    def steps_to_dataframe(steps: List[WerStep]) -> pd.DataFrame:
        arr = np.array([
            WerUtil._wer_step_to_pandas_row_lit(step)
            for step in steps
        ])
        return pd.DataFrame(
            arr,
            columns=['step_type', 'reference', 'hypothesis', 'cost']
        )
